# cvache
# 10/6/21
# ingestor.py
from riotwatcher import LolWatcher, ApiError
from .Config import Config


class Ingestor():
    def __init__(self, userConfig: Config):
        self.RIOT_API = LolWatcher(api_key=userConfig.getAPI_Key())
        self.REGION_NA1 = 'na1'
        self.REGION_AMERICAS= 'americas'





    #final function to call
    #returns an array of matchIds of size listSize of matchIds
    def createDownloadQueue(self, seedUsername, listSize=1000):
        downloadQueue = []
        matchlist = self.getMatchlist(self.getPuuidFromUsername(seedUsername))
        for match in matchlist:
            downloadQueue.append(match)
        
        while len(downloadQueue) < listSize:
            seedMatch = downloadQueue.pop()
            newUUIDS = self.getUUIDs(seedMatch)
            downloadQueue.append(seedMatch)
            for uuid in newUUIDS:
                for match in self.getMatchlist(uuid):
                    downloadQueue.append(match)
        
        return downloadQueue


    #----- Create download queue -----#
    # start with seed username
    # get uuid for the seed user
    # get matchlist for seed user
    # add matchIDs in matchlist to match download queue

    # once queue gets to len x...

    #----- Parse download -----#

    '''
    using tier and division, downloads matches for that rank until the program is stopped


    tier (string) - the tier to query, i.e. DIAMOND
    division (string) - the division to query, i.e. III
    page (int) - the page for the query to paginate to. Starts at 1.
    '''

    def createDownloadQueueV2(self, tier, division, page):

        #get list of summonerIds for specified tier and div
        leagueData = self.RIOT_API.league.entries(self.REGION_NA1, 'RANKED_SOLO_5x5', tier, division, page=page)
        summonerIds = []
        for summoner in leagueData:
            summonerIds.append(summoner['summonerId'])

        #use list of summonerIds to generate a puuid for each summonerId
        puuids = []
        for id in summonerIds:
            puuids.append( self.RIOT_API.summoner.by_id(self.REGION_NA1, id)['puuid'] )

        return self.matchlistFromPuuids(puuids)
        
        


    #using a list of puuids, generate a matchlist
    def matchlistFromPuuids(self, puuidList):

        matchList = []
        for puuid in puuidList:
            try:  
                matchList = matchList + self.RIOT_API.match.matchlist_by_puuid(  
                    self.REGION_AMERICAS, 
                    puuid, 
                    #Begin time currently unsupported by riotwatcher
                    #begin_time=float(1633546800), #12:00 PM on 10/6/21, start of patch 11.20
                    type='ranked',
                    start=0,
                    count=20
                )
            except ApiError:
                print('[ERROR] matchlist generation error. Bad puuid: ', puuid)
        return matchList
            
        
    #Using a username, returns the UUID for that username
    def getPuuidFromUsername(self, username):
        return self.RIOT_API.summoner.by_name(self.REGION_NA1, username)['puuid']

    #Using UUID, returns the last 20 matches for a given uuid
    def getMatchlist(self, puuid):
        return self.RIOT_API.match.matchlist_by_puuid(
            self.REGION_AMERICAS, 
            puuid, 
            type='ranked',
            start=0, 
            count=20
        )

    #Using a matchID, gets the match and returns the 10 uuids from the game
    def getUUIDs(self, matchId):
        uuids = []
        matchInfo = self.RIOT_API.match.by_id(region=self.REGION_AMERICAS, match_id=matchId)
        for participant in matchInfo['metadata']['participants']:
            uuids.append(participant)
        return uuids




# cvache
# 10/6/21
# upload.py


import json

#takes a single match and uploads it to DB
#def uploadSingleMatch(matchData, rankInfo=None):
    #dynamoDB = boto3.resource('dynamodb', endpoint_url=config('DYNAMO'))
    #matchData = cleanData(matchData, rankInfo)

    #matchesTable = dynamoDB.Table('matches')

    #matchesTable.put_item(Item=matchData)
        
#TODO:
#takes a set of matches and uploads them to DB
# def uploadSetOfMatches(matches)


'''
    create match table in Dynamo. 

    Helper function for uploadSingleMatch()
'''
# def initMatchTable():
#     dynamoDB = boto3.resource('dynamodb', endpoint_url=config('DYNAMO'))

#     matchesTable = dynamoDB.create_table(
#         TableName='matches',
#         KeySchema=[
#             {
#                 'AttributeName': 'matchId',
#                 'KeyType': 'HASH'
#             }
#         ],
#         AttributeDefinitions=[
#             {
#                 'AttributeName': 'matchId',
#                 'AttributeType': 'S'
#             }
#         ],
#         ProvisionedThroughput={
#             'ReadCapacityUnits': 10,
#             'WriteCapacityUnits': 10
#         }
#     )

#     return matchesTable

# initMatchTable()

#Cleans match data 
'''
Takes matchId and exposes it to root of data
Adds rankInfo to root

'''


def downloadLocal(matchData, rankInfo=None):
    #matchData = cleanData(matchData)

    with open("matches/" + str(matchData['matchId']) + ".json") as file:
        file.write(json.dumps(matchData, indent=4))




from .Config import Config
from .DownloadController import DownloadController
from .Ingestor import Ingestor
from .LoadS3 import LoadS3
from .LocalDownload import LocalDownload

__all__ = [
    "Config",
    "DownloadController",
    "Ingestor",
    "S3Loader"
]


from .Download import Download
import boto3
import json
import os

class LoadS3(Download):

    def __init__(self, bucket):
        self.bucket = bucket

    # def download(self, matchData):
    #     self.upload_file(self, matchData)

    def download(self, matchData):
        """Upload a file to an S3 bucket

        :param matchData: File to upload
        :param bucket: Bucket to upload to
        :param object_name: S3 object name. If not specified then file_name is used
        :return: True if file was uploaded, else False
        """

        object_name = str(matchData['matchId']) + ".json"
        
        #write file
        with open(object_name, "w") as f:
            f.write( json.dumps( matchData, indent=4 ) )


        # Upload the file
        s3_client = boto3.client('s3')
        with open(object_name, "rb") as f:
            s3_client.upload_fileobj(f, self.bucket, object_name)
        
            # print("Error")
            # #Delete file
            # os.remove(object_name)


            #return False

        # Delete file
        os.remove(object_name)
        return True


from .Download import Download
import json


class LocalDownload(Download):
    def __init__(self):
        pass

    def hello():
        return "hello from download"

    

    '''
    Download matches to local (matches) folder

    match
    
    '''
    def download(self, matchData):
        with open("./matches/" + str(matchData['matchId']) + ".json", mode="w") as file:
            file.write( json.dumps( matchData, indent=4 ) )  


# cvache
# 10/6/21
# downloader.py

from .Ingestor import Ingestor, ApiError
from .Download import Download


class DownloadController:
    
    def __init__(self, downloadConfig: Download, ingestorEngine: Ingestor):

        self.downloadConfig = downloadConfig
        self.REGION_AMERICAS = 'americas'
        self.ing = ingestorEngine


    #downloads matches infintly, stop by keyboard interrupt
    def downloadInfinite(self, tier, division):
        
        page = 1
        try:
            while(True):
                rankInfo = {
                    'tier' : tier,
                    'division' : division
                }
                print('[INFO] Downloading matches from ', tier, ' ', division, 'Page ', page)
                matchlist = self.ing.createDownloadQueueV2(tier, division, page)
                print('[INFO] Finished building list. Starting download...')
                self.downloadConfig.downloadMatchlist(matchlist, rankInfo=rankInfo)
                print('[INFO] Download finished.')
                page += 1


        except KeyboardInterrupt:
            print('Program ended')
            print('Tier: ', tier)
            print('Division: ', division)
            print('Current page: ', page)


    '''
    using a seed username and size, downloads that many matches

    seedUser (string) - a username to use as a seed i.e. Kascadian
    size (int) - the amount of matches to download
    '''
    def downloadFixed(self, seedUser, size):

        print('[INFO] Starting list creation')
        list = self.ing.createDownloadQueue(seedUser, listSize=size)
        print('[INFO] Finished creating list')

        self.downloadMatchlist(list)

    '''
    Given a matchlist, parse and clean it
    sends every match to another funtion, thus not breakign the stackflow
    
    matchlist (list) - list of matchIds to download
    '''
    def downloadMatchlist(self, matchlist, rankInfo=None):
        for match in matchlist:
            isMatch = True
            try:
                matchData = self.ing.RIOT_API.match.by_id(self.REGION_AMERICAS, match)
            except ApiError:
                print("[ERROR1] Error in downloading match ", match)
                isMatch=False
            except ConnectionError:
                print("[ERROR2] Error in downloading match ", match)
                isMatch=False
            
            if isMatch:
                #move matchId from metadata to root of object to expose for use as hash key
                matchData['matchId'] = matchData['metadata']['matchId']
                matchData['rankInfo'] = rankInfo
                del matchData['metadata']['matchId']

                #this needs to be a function call
                self.downloadConfig.download( matchData )
            else:
                #not sure if I need this but my c++ says add it anyway
                del matchData


from setuptools import setup, find_packages
import os.path

curdir = os.path.dirname(__file__)

readmePath = os.path.join(curdir, "README.md")

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name = 'matchfarm',
    version = '0.1.10',
    author = 'Carson Vache',
    author_email = 'carsonvache@gmail.com',
    license = 'GNU GENERAL PUBLIC LICENSE',
    description = 'Farm LOL matches',
    long_description = open(readmePath).read(),
    long_description_content_type = "text/markdown",
    url = 'https://github.com/cvache/LeagueMatchFarm',
    #py_modules = ['matchFarm', 'src'],
    packages = find_packages("src"),
    package_dir = {"": "src",},
    install_requires = [
        "requests",
        "riotwatcher",
        "boto3",
        ],
    python_requires='>=3.7',
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "Operating System :: OS Independent",
    ]
)